/*
 * PC-Show - A crossplatform imageviewer, including integrated 
 * functionality with PC-Praxis.
 *
 * Copyright (C) 2004  Bent Bisballe Jensen
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include "global.h"

#include "filecache.h"

#include <wx/dir.h>



FileCache::FileCache(Watchfile* wf)
{
AA
	watchfile = wf;
	filecache = NULL;
}

FileCache::~FileCache()
{
AA
	if(filecache) delete filecache;
}

wxArrayString *FileCache::getFileList()
{
AA
	if(!filecache) check();	
	return filecache;
}

bool FileCache::isImage(const wxChar* cfname)
{
AA
	wxString fname = (new wxString(cfname))->Lower().AfterLast('.');
	wxString *str = new wxString(wxImage::GetImageExtWildcard());
	wxString *oldstr;
	
	while((str->AfterFirst('.')).IsEmpty() == false) {
		oldstr = str;
		str = new wxString(str->AfterFirst('.'));
		delete oldstr;
		if(fname.CompareTo(str->BeforeFirst(';')) == 0) {
			delete str;
			return true;
		}
	}
	return false;
}

bool FileCache::compare(wxArrayString *a1, wxArrayString *a2)
{
AA
	if(a1->Count() != a2->Count()) return true;
	
	for(unsigned int cnt = 0; cnt < a1->Count(); cnt++) {
//		DEBUGPRINTPARPAR("+ [FileCache::compare]\n\t\t\"%s\"\t\t\"%s\"\n\t\t\t-> ",
//			a1->Item(cnt).c_str(),
//			a2->Item(cnt).c_str())
		if(a1->Item(cnt).CompareTo(a2->Item(cnt)) != 0) {
//			DEBUGPRINT("FAILED!\n")
			return true;
		}
//		DEBUGPRINT("success!\n")
	}
	return false;
}


/** check()
 * Checks the cache, and updates it. Return true if changed.
 */
bool FileCache::check()
{
AA
	if(!watchfile->hasFile()) return false;
	
	// The file points at an unexisting folder...
	if(!wxDirExists(watchfile->getPath()->c_str())) return false;

	wxArrayString *files = new wxArrayString();
	wxDir::GetAllFiles(watchfile->getPath()->c_str(), files, wxEmptyString, wxDIR_FILES);
	
	// Remove all non-image files from the list
	for(unsigned int cnt = 0; cnt < files->Count(); cnt++) {
		files->Item(cnt).Replace("\\", "/");
		if(!isImage(files->Item(cnt))) {
			files->RemoveAt(cnt);
			// Another item has now taken position cnt, mut check it also.
			cnt--;
		}
	}

	if(!filecache) {
		filecache = files;
		return true;
	}
	
	// FIXME Check on more than just filenames!
	bool changed = compare(filecache, files);

	delete filecache;
	filecache = files;
	
	// Retrun true if the cache has changed.
	return changed;
}


/** clear()
 * Clears the cache.
 */
void FileCache::clear()
{
AA
	if(filecache) delete filecache;
	filecache = NULL;
}
