/*
 * PC-Show - A crossplatform imageviewer, including integrated 
 * functionality with PC-Praxis.
 *
 * Copyright (C) 2004  Bent Bisballe Jensen
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
#include "global.h"

#include "fileselector.h"
#include "mainwindowsplitter.h"
#include "mainwindow.h"

#include <wx/dir.h>
#include <wx/list.h>

#include "options.h"

BEGIN_EVENT_TABLE(FileSelector, wxListView)
//	EVT_MENU(KEY_UP, FileSelector::moveUp)
//	EVT_MENU(KEY_DOWN, FileSelector::moveDown)

	EVT_KEY_DOWN(FileSelector::evtOnKeyDown)
	
//	EVT_KILL_FOCUS(FileSelector::evtOnLostFocus)

	EVT_LEFT_DCLICK(FileSelector::evtOnDblClick)
END_EVENT_TABLE()


FileSelector::FileSelector(wxWindow* parent)
        : wxListView(parent)//, -1, wxDefaultPosition, wxDefaultSize, wxFULL_REPAINT_ON_RESIZE)
{
AA
	imagelist = new wxImageList(32, 32);

	// Fix! Must add an 32x32 icon to make the listcontrol use this size!
	wxImage *tnimage = new wxImage(DEFAULT_ICON, wxBITMAP_TYPE_ANY, -1);
	if(!tnimage->Ok()) {
		fprintf(stderr, "FATAL: Missing default icon: %s!\n", DEFAULT_ICON);
		exit(1);
	}
	wxBitmap *dummy = new wxBitmap(tnimage, -1);
	imagelist->Add(*dummy);
	delete tnimage;
	delete dummy;

	SetImageList(imagelist, wxIMAGE_LIST_SMALL);
	
	startShiftSel = -1;

	// Initialize an empty list.
	clear();
}

FileSelector::~FileSelector() 
{
AA
	if(imagelist) delete imagelist;	
//	delete images;
}


/** Functions to control the keyboard navigation in the fileselector:
 */
void FileSelector::removeSelection()
{AA
	for(int sel = 0; sel < GetItemCount(); sel++) {
		Select(sel, false);
	}
}

void FileSelector::space(bool shift, bool ctrl)
{AA
	SetFocus();

	bool issel;
	int sel = GetFocusedItem();
	
#ifdef __WXMSW__
	// There is a bug in IsSelected() on windows... always returns true!
	issel = false;
	int tst = GetFirstSelected();
	while(tst != -1) {
		if(tst == sel) issel = true;
		tst = GetNextSelected(tst);
	}
#else
	issel = IsSelected(sel);
#endif /*__WXMSW__*/
	
	DEBUGPRINTPAR("SPACE! %d\n", issel)

	if(!ctrl && !shift) removeSelection();
	Select(sel, !issel);
}

void FileSelector::up(bool shift, bool ctrl)
{AA
	SetFocus();

	int sel = GetFocusedItem();
	if(sel == 0) return;
	sel--;
	Focus(sel);
	
	
	if(shift) {
		if(startShiftSel == sel + 1) {
			if(!ctrl) removeSelection();
		}

		Select(sel + 1, true);
		Select(sel, true);

		if(sel >= startShiftSel) Select(sel + 1, false);

		while(sel != startShiftSel) {
			Select(sel, true);
			if(sel < startShiftSel) sel++;
			else sel--;
		}
	} else  startShiftSel = sel;
}

void FileSelector::down(bool shift, bool ctrl)
{AA
	SetFocus();
	
	int sel = GetFocusedItem();
	if(sel == GetItemCount() - 1) return;
	sel++;
	Focus(sel);
	
	if(shift) {
		if(startShiftSel == sel - 1) {
			if(!ctrl) removeSelection();
		}

		Select(sel - 1, true);
		Select(sel, true);

		if(sel <= startShiftSel) Select(sel - 1, false);

		while(sel != startShiftSel) {
			Select(sel, true);
			if(sel < startShiftSel) sel++;
			else sel--;
		}
	} else  startShiftSel = sel;
}	

void FileSelector::evtOnKeyDown(wxKeyEvent& event)
{AA
	switch(event.m_keyCode) {
		case WXK_SPACE:
			space(event.m_shiftDown, event.m_controlDown);
			break;
		
		case WXK_UP:
			up(event.m_shiftDown, event.m_controlDown);
			break;
		
		case WXK_DOWN:
			down(event.m_shiftDown, event.m_controlDown);
			break;
		
		case WXK_LEFT:
			mainwindow->imagematrix->left();
			break;
		
		case WXK_RIGHT:
			mainwindow->imagematrix->right();
			break;
		
//		case WXK_RETURN:
//			break;

	}
//	event.Skip();
}

void FileSelector::evtOnLostFocus(wxFocusEvent& WXUNUSED(event) )
{AA
	//SetFocus();
}

void FileSelector::evtOnDblClick(wxMouseEvent& WXUNUSED(event) )
{
AA
	MainWindow* main = (MainWindow*)(GetParent()->GetParent());
	main->showSelection();
}


void FileSelector::clear() 
{
AA
	//
	// Emptying the selectiontable
	//
	while(GetItemCount() > 0) {
		FileSelectorItem* item = (FileSelectorItem*)GetItemData(0);
		DeleteItem(0);
		delete item;
	}

	ClearAll();
	imagelist->RemoveAll();

	InsertColumn(0, "Dato", wxLIST_FORMAT_LEFT, -1);
	InsertColumn(0, "Fil", wxLIST_FORMAT_LEFT, -1);
}

void FileSelector::refresh(FileCache* filecache) 
{
AA	
	wxArrayString *files = filecache->getFileList();
	mainwindow->clearIdleEventQueue();

	// Delete items corresponding to deleted files.
	for(int cnt = 0; cnt < GetItemCount(); cnt++) {
		FileSelectorItem* item = (FileSelectorItem*)GetItemData(cnt);
		bool gone = true;
		
		for(unsigned int strcnt = 0; strcnt < files->Count(); strcnt ++) {
			if(files->Item(strcnt).CompareTo(item->getPath()->c_str()) == 0) {
				gone = false;
			}
		}

		if(gone) {
			// FIXME: Delete the thumbnailfile if it exists.
			DeleteItem(cnt);
			// Another item now populates seat nr. cnt... must that one also.
			cnt--;
			delete item;
		}
	}
	
	// Check if the files in the list has been inserted into the control.
	for(unsigned int cnt = 0; cnt < files->Count(); cnt++) {
		bool missing = true;
		
		for(int strcnt = 0; strcnt < GetItemCount(); strcnt++) {
			FileSelectorItem* item = (FileSelectorItem*)GetItemData(strcnt);

			if(files->Item(cnt).CompareTo(item->getPath()->c_str()) == 0) missing = false;
		}		
		
		if(missing) {
			FileSelectorItem *item;
			wxListItem *li;
			Image *img;
	
			wxString *filename = new wxString(files->Item(cnt));
			img = new Image(*filename);
			delete filename;
	
			item = new FileSelectorItem(img);
			item->setIndex(imagelist->Add(*item->getThumbnailOnLoad(mainwindow)));
			
			li = new wxListItem();
			li->SetImage(item->getIndex());
			li->SetText(item->getName()->c_str());
			li->SetData(item);
	
			
			InsertItem(*li);
			delete li;
			SetItem(0, 1, item->getDate()->c_str());
		}
	}

//	DEBUGPRINT("+ [FileSelector::refresh] colour\n")
	// Colour switching variable
	int grey = 0;
	
	for(int cnt = 0; cnt < GetItemCount(); cnt++) {
		if(grey) SetItemBackgroundColour(cnt, wxColour(235, 235, 235));
		else SetItemBackgroundColour(cnt, wxColour(255, 255, 255));
		grey = 1 - grey;
	}

	SetColumnWidth(0, wxLIST_AUTOSIZE);
	SetColumnWidth(1, wxLIST_AUTOSIZE);
	
	((MainWindowSplitter*)GetParent())->SetSashPosition(GetColumnWidth(0) + GetColumnWidth(1) + 22);

	// Set focus to the first item, and the listctrl.
	Focus(0);
	SetFocus();
//	DEBUGPRINT("+ [FileSelector::refresh] done\n")
}

/* // The old one!
void FileSelector::refresh(FileCache* filecache) 
{
AA
	// Colour switching variable
	int grey = 0;
	
	wxArrayString *files = filecache->getFileList();
	
	SetImageList(imagelist, wxIMAGE_LIST_SMALL);
	
	InsertColumn(0, "Dato", wxLIST_FORMAT_LEFT, -1);
	InsertColumn(0, "Fil", wxLIST_FORMAT_LEFT, -1);
	
	MainWindow *main = (MainWindow*)(GetParent()->GetParent());

	wxListItem *li;
	Image *img;
	for(unsigned int cnt = 0; cnt < files->Count(); cnt++) {
		if(isImage(files->Item(cnt))) {
			FileSelectorItem *item;

			wxString *filename = new wxString(files->Item(cnt));
			img = new Image(*filename);
			delete filename;

			item = new FileSelectorItem(img);
			item->setIndex(imagelist->Add(*item->getThumbnailOnLoad(main)));
			
			li = new wxListItem();
			li->SetImage(item->getIndex());
			li->SetText(item->getName()->c_str());
			li->SetData(item);

			
			grey = 1 - grey;
			if(grey) li->SetBackgroundColour(wxColour(235, 235, 235));
			else li->SetBackgroundColour(wxColour(255, 255, 255));
			InsertItem(*li);
			delete li;
			SetItem(0, 1, item->getDate()->c_str());
		}
	}

	SetColumnWidth(0, wxLIST_AUTOSIZE);
	SetColumnWidth(1, wxLIST_AUTOSIZE);
	
	((MainWindowSplitter*)GetParent())->SetSashPosition(GetColumnWidth(0) + GetColumnWidth(1) + 22);

	// Set focus to the first item, and the listctrl.
	Focus(0);
	SetFocus();
}
*/
void FileSelector::loadNextThumbnail(FileSelectorItem *item)
{
AA	
	wxBitmap *thumbnail = item->getThumbnail();

	// If thumbnail == NULL means that the real thumbnail has already been
	//	loaded, or that the imagefile was corrupted.
	if(thumbnail) {
		imagelist->Replace(item->getIndex(), *thumbnail);
		Refresh(true);
	}
}
