/*
 * PC-Show - A crossplatform imageviewer, including integrated 
 * functionality with PC-Praxis.
 *
 * Copyright (C) 2004  Bent Bisballe Jensen
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
#include "global.h"

#include "fileselectoritem.h"

#include "mainwindow.h"


#include <wx/filename.h>
#include <wx/dir.h>
#include <wx/file.h>

FileSelectorItem::FileSelectorItem(Image* img)
{AA
	image = img;

	// FIXME.... this is ugly!!!
	wxString str = wxString(*image->getPath());
	str.Replace("\\", "/");
	wxString dir = image->getPath()->BeforeLast('/');
	thumbnailpath = new wxString(dir + '/' + THUMBNAILDIR + '/');
	
	thumbnailfile = new wxString(*thumbnailpath + *getName() + ".png");

	thumbnail = NULL;
	index = -1;
}

FileSelectorItem::~FileSelectorItem() 
{AA
	if(thumbnail) delete thumbnail;
	delete thumbnailpath;
	delete thumbnailfile;
		
	if(image->isInUse() == false) delete image;
}

void FileSelectorItem::setIndex(int idx)
{AA
	index = idx;
}

int FileSelectorItem::getIndex()
{AA
	return index;
}

Image *FileSelectorItem::getImage() 
{AA
	return image;
}

wxString *FileSelectorItem::getName() 
{AA
	return image->getName();
}

wxString *FileSelectorItem::getPath() 
{AA	
	return image->getPath();
}

wxString *FileSelectorItem::getDate() 
{AA
	return image->getDate();
}

  //////////////////////
 // thumbnail methods
//////////////////////

wxBitmap *FileSelectorItem::getThumbnail()
{AA
	if(thumbnail) return NULL; // The real thumbnail has already been loaded.

	wxLogNull noLogging; // Disable noisy errors

	//creating
	DEBUGPRINTPAR("+ [FileSelectorItem::getThumbnail] Creating %s\n", thumbnailfile->c_str())

	wxImage *img = image->getImage();

	// If img == NULL, it means that the imagefile was corrupted.
	if(!img) return NULL;

	if(!img->Ok()) {
		fprintf(stderr, "FATAL: Thumbnail is not a valid image... this should not happen!\n");
		exit(1);
		return NULL;
	}

	wxImage *tnimage = new wxImage(img->Scale(32, 32));

	if(!wxDir::Exists(*thumbnailpath)) wxMkdir(*thumbnailpath);
	tnimage->SaveFile(*thumbnailfile, wxBITMAP_TYPE_PNG);
	thumbnail = new wxBitmap(tnimage, -1);
	delete tnimage;

	// The image was connected, when the request for a thumbnail generation was made.
	image->disconnect();
	return thumbnail;
}


wxBitmap *FileSelectorItem::getThumbnailOnLoad(wxWindow *main)
{AA
	wxLogNull noLogging;
	// A thumbnail has already been loaded
	if(thumbnail) return thumbnail;
	
	// No thumbnail loaded, but the tn file exists
	if(wxFile::Exists(*thumbnailfile)) {
		wxImage *tnimage = new wxImage();
		if(tnimage->LoadFile(*thumbnailfile, wxBITMAP_TYPE_ANY, -1) && tnimage->Ok()) {
			thumbnail = new wxBitmap(tnimage, -1);
			delete tnimage;
			return thumbnail;
		}

		// If the thumbnail is corrupted, delete it, and request regeneration.
		wxRemoveFile(*thumbnailfile);
	}
	
	// No tn loaded and no tn file exists
	wxImage *tnimage = new wxImage(DEFAULT_ICON, wxBITMAP_TYPE_ANY, -1);
	if(!tnimage->Ok()) {
		fprintf(stderr, "FATAL: Missing or corrupted default icon: %s!\n", DEFAULT_ICON);
		exit(1);
	}
	wxBitmap *dummy = new wxBitmap(tnimage, -1);

	// Connect to the image, so it will not be freed before the thumbnail has been generated.
	image->connect();

	((MainWindow*)main)->insertIdleEvent(LOAD_THUMBNAIL, this);
	delete tnimage;
	return dummy;
	
}

/*
	MainWindow *main = (MainWindow*)(fileselector->GetParent()->GetParent());
	main->insertIdleEvent(LOAD_THUMBNAIL, item);
*/
