/*
 * PC-Show - A crossplatform imageviewer, including integrated 
 * functionality with PC-Praxis.
 *
 * Copyright (C) 2004  Bent Bisballe Jensen
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "global.h"

#include "fswindow.h"

#include "staticimage.h"

BEGIN_EVENT_TABLE(FSWindow, wxFrame)
//	EVT_KILL_FOCUS(FSWindow::evtOnLostFocus)
	EVT_CLOSE(FSWindow::evtOnClose)
	EVT_KEY_DOWN(FSWindow::evtOnKey)
	EVT_LEFT_DCLICK(FSWindow::evtOnMouseDblClick)
END_EVENT_TABLE()


FSWindow::FSWindow(Image* img): 
#ifdef __WXMSW__
	// No need to hide border in windows
	wxFrame(NULL, -1, "Fullscreen")
#else
	// Hide border in linux, for fullscreen emulation
	wxFrame(NULL, -1, "Fullscreen", wxPoint(-1, -1), wxSize(-1,-1), wxNO_BORDER) 
#endif
{AA	
	image = img;
	image->connect();
	
	int w, h;
	
	wxDisplaySize(&w, &h);
	
#ifdef __WXMSW__
	// Fullscreen in windows
	ShowFullScreen(true);
#else
	// Fullscreen on linux
	SetSize(0,0,w,h);
	Show(true);
#endif

	wxImage *wximg = image->getImage();

	// If img == NULL it means that the imagefile was corrupted.
	if(!wximg) return;
	
	double fimgw = (double)wximg->GetWidth();
	double fimgh = (double)wximg->GetHeight();
	
	double fh = (double)h;
	double fw = (double)w;

	double scalew = fimgw / fw;
	double scaleh = fimgh / fh;

	double scale;
	if(scalew > scaleh) scale = scalew;
	else scale = scaleh;

	int width = (int)(fimgw/scale);
	int height = (int)(fimgh/scale);
	
	int x = (w - width)/2;
	int y = (h -height)/2;
	
	bmp = new wxBitmap(wximg->Scale(width, height));

	StaticImage *simg = new StaticImage(this, -1, *bmp);
	simg->SetSize(width, height);
	simg->Move(x, y);
	simg->SetBitmap(*bmp);

	DEBUGPRINTPARPAR("+ [FSWindow::FSWindow] Image (x,y)%d,%d", (w - width)/2, (h -height)/2 )
	DEBUGPRINTPARPAR(" - (w,h)%d,%d\n", width, height)

	delete bmp;
}

FSWindow::~FSWindow() 
{AA
	
	image->disconnect();
}

void FSWindow::evtOnMouseDblClick(wxMouseEvent& event)
{AA
	Destroy();
	event.Skip();
}

void FSWindow::evtOnClose(wxCloseEvent& event) 
{AA
	Destroy();
	event.Skip();
}

void FSWindow::evtOnLostFocus(wxFocusEvent& event) 
{AA
	Destroy();
	event.Skip();
}

void FSWindow::evtOnCommand(wxCommandEvent& event) 
{AA
//	Show(false);
	Destroy();
	event.Skip();
}

void FSWindow::evtOnKey(wxKeyEvent& event) 
{AA
//	Show(false);
	if(event.GetKeyCode() == WXK_ESCAPE) Destroy();
	if(event.GetKeyCode() == 'Z') Destroy();
	event.Skip();
}
