/*
 * PC-Show - A crossplatform imageviewer, including integrated 
 * functionality with PC-Praxis.
 *
 * Copyright (C) 2004  Bent Bisballe Jensen
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
#include "global.h"

#include "image.h"
#include "fswindow.h"
#include "mainwindow.h"

#include <wx/filename.h>
#include <wx/dir.h>
#include <wx/file.h>

Image::Image(wxString& file)
{AA
	file.Replace("\\", "/");
	
	path = new wxString(file);
	name = new wxString(path->AfterLast('/'));

//	DEBUGPRINTPAR("->[Image::Image] Creating Image from path: [%s]\n", file.c_str())
	
	wxFileName *fn = new wxFileName(*path);
	wxDateTime *dt = new wxDateTime();
	fn->GetTimes(NULL, NULL, dt);	

	date = new wxString(dt->FormatISODate());
#ifndef __WXMSW__
	date->Append("\n");
#else
	date->Append(" ");
#endif
	date->Append(dt->FormatISOTime().c_str());
	
	delete dt;
	delete fn;
	
	usage = 0;
	
	image = NULL;
}

Image::~Image() 
{AA
	if(image) delete image;
	if(path) delete path;
	if(name) delete name;
	if(date) delete date;
}

void Image::connect()
{AA
	usage++;
}

void Image::disconnect()
{AA
	usage--;
	if(!usage) {
		delete image;
		image = NULL;
	}
}

bool Image::isInUse()
{AA
	return (usage != 0);
}

void Image::showFullscreen()
{AA
	FSWindow *fsw = new FSWindow(this);
	fsw->Show(true);
}

/*
void Image::addComment(wxString *comment) 
{AA
}

void Image::addNote(wxString *note, wxPoint *point) 
{AA
}
*/

wxImage *Image::getImage() 
{AA	
	// Disable error messages.
	wxLogNull noLogging;

	if(!usage) {
		fprintf(stderr, "FATAL: Tried to read image, without connecting.\n");
		exit(1);
	}
	if(!image) {
#ifdef __WXMSW__
		wxBusyCursor loading;
#endif
		DEBUGPRINTPAR("+ [Image::getImage] Loading file: %s\n", path->c_str())
		image = new wxImage();
		if(wxFileExists(path->c_str())) {
			DEBUGPRINT(" * File exists...Loading\n")
			image->LoadFile(path->c_str(), wxBITMAP_TYPE_ANY, -1);
		}

		if(!image->Ok()) {
			DEBUGPRINT(" * Corrupted... renaming and removing\n")
			wxString newpath(*path);
			newpath.Append(BADFILE_EXTENSION);
			wxRenameFile(*path, newpath);

			delete image;
			image = NULL;
		}

/*		
		else {
			DEBUGPRINT(" * File does not exists...Loading default\n")
			image->LoadFile(DEFAULT_ICON, wxBITMAP_TYPE_ANY, -1);
		}

		if(!image->Ok()) {
			DEBUGPRINT(" * Renaming file\n")
			usage = 10000; // Now the empty imagedata will never be freed! (Prevents crash)
			image->LoadFile(DEFAULT_ICON, wxBITMAP_TYPE_ANY, -1);
			wxString newpath(*path);
			newpath.Append(BADFILE_EXTENSION);

			wxRenameFile(*path, newpath);

			char msg[512];
			char title[256];
			sprintf(msg, "Filen %s er beskadiget.\nDen omdoebes til %s", 
				path->c_str(),
				newpath.c_str());
			sprintf(title, "Kan ikke indlaese %s", 
				path->AfterLast('/').c_str());
			wxMessageDialog *msgdlg = new wxMessageDialog(NULL, (const char*)msg, (const char*)title, wxOK);
			msgdlg->ShowModal();
			mainwindow->refresh();
		}
*/
	}

	return image; 
}

wxString *Image::getName() 
{AA
	return name; 
}

wxString *Image::getPath() 
{AA
	return path; 
}

wxString *Image::getDate() 
{AA
	return date; 
}

Comment::Comment() 
{AA
}

Comment::~Comment() 
{AA
}

void Comment::remove() 
{AA
}
