/*
 * PC-Show - A crossplatform imageviewer, including integrated 
 * functionality with PC-Praxis.
 *
 * Copyright (C) 2004  Bent Bisballe Jensen
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "global.h"

#include "imagematrix.h"
#include "mainwindow.h"

#include "options.h"

BEGIN_EVENT_TABLE(ImageMatrix, wxPanel)
//	EVT_MENU(KEY_LEFT, ImageMatrix::moveLeft)
//	EVT_MENU(KEY_RIGHT, ImageMatrix::moveRight)

	EVT_KEY_DOWN(ImageMatrix::evtOnKeyDown)

	EVT_TOOL(IMGMTX_INSERT_ITEM, ImageMatrix::evtOnInsertImage)
//	EVT_SET_FOCUS(ImageMatrix::evtOnGotFocus)

	EVT_SIZE(ImageMatrix::evtOnSize)
	EVT_LEFT_DOWN(ImageMatrix::evtOnMouseClick)
	EVT_LEFT_DCLICK(ImageMatrix::evtOnMouseClick)
	EVT_BUTTON(BTN_CLOSE, ImageMatrix::evtOnBtnCloseClick)
END_EVENT_TABLE()

ImageMatrix::ImageMatrix(wxWindow* parent) : 
	wxPanel(parent, -1, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxFULL_REPAINT_ON_RESIZE)
{AA
	images = NULL;
}

ImageMatrix::~ImageMatrix()
{AA
}

/** Functions to control the keyboard navigation in the fileselector:
 */
void ImageMatrix::left()
{AA
	ImageMatrixItem *node;
	ImageMatrixItem *prev = NULL;
	
	if(images == NULL) return;
	if(images->getSize() == 0) return;
	node = (ImageMatrixItem*)images->startIteration();
	while(node) {
		if(node->isSelected()) {
			if(!prev) return;
			prev->setSelected(true);
			node->setSelected(false);
			return;
		}
		prev = node;
		node = (ImageMatrixItem*)images->next();
	}
}

void ImageMatrix::right()
{AA
	ImageMatrixItem *node;
	ImageMatrixItem *next = NULL;
	
	if(images == NULL) return;
	if(images->getSize() == 0) return;
	node = (ImageMatrixItem*)images->startIterationReversed();
	while(node) {
		if(node->isSelected()) {
			if(!next) return;
			next->setSelected(true);
			node->setSelected(false);
			return;
		}
		next = node;
		node = (ImageMatrixItem*)images->prev();
	}
}

void ImageMatrix::evtOnKeyDown(wxKeyEvent& event)
{AA
	switch(event.m_keyCode) {
		case WXK_SPACE:
			mainwindow->fileselector->space(event.m_shiftDown, event.m_controlDown);
			break;
		
		case WXK_UP:
			mainwindow->fileselector->up(event.m_shiftDown, event.m_controlDown);
			break;
		
		case WXK_DOWN:
			mainwindow->fileselector->down(event.m_shiftDown, event.m_controlDown);
			break;
		
		case WXK_LEFT:
			left();
			break;
		
		case WXK_RIGHT:
			right();
			break;
		
//		case WXK_RETURN:
//			break;
	}
//	event.Skip();
}

void ImageMatrix::clear()
{AA
	if(!images) return;

	ImageMatrixItem *item = (ImageMatrixItem*)images->getFirst();
	while(item) {
		removeImage(item);
		item = (ImageMatrixItem*)images->getFirst();
	}
}
/*
void ImageMatrix::evtOnGotFocus(wxFocusEvent& WXUNUSED(event) )
{
	mainwindow->fileselector->SetFocus();
}
*/



ImageMatrixItem *ImageMatrix::getSelected() 
{AA
	ImageMatrixItem *node;
	
	if(images == NULL) return NULL;
	if(images->getSize() == 0) return NULL;
	node = (ImageMatrixItem*)images->startIteration();
	while(node != NULL) {
		if(node->isSelected()) {
			return node;
		}
		node = (ImageMatrixItem*)images->next();
	}
	return NULL;
}

void ImageMatrix::addImage(ImageMatrixItem *mtxitm)
{AA
	Option *optcols = options.getOption("ImageMatrix.cols");
	Option *optrows = options.getOption("ImageMatrix.rows");

	if(images == NULL) images = new LinkedList();

	ImageMatrixItem *tstimg = (ImageMatrixItem*)images->startIteration();

	mtxitm->getImage()->connect();
	wxImage *mtxwximg = mtxitm->getImage()->getImage();

	// If mtxwximg == NULL it means that the imagefile is corrupted.
	if(mtxwximg) {
		while(tstimg) {
			if(tstimg->getImage() ==  mtxitm->getImage()) {
				removeImage(tstimg);
			}
			tstimg = (ImageMatrixItem*)images->next();
		}
		images->insert(mtxitm);
		
		if(images->getSize() > (optrows->getInt() * optcols->getInt())) {
			removeImage((ImageMatrixItem*)images->getLast());
		}
		
		if(!mtxitm->GetParent()) mtxitm->Create(this, -1, wxDefaultPosition, wxDefaultSize, wxSIMPLE_BORDER);
	
		mtxitm->pack();
	
		mtxitm->Show(true);
	
		setSelection(mtxitm);
	
		setSizes();
	}
	mtxitm->getImage()->disconnect();
}

void ImageMatrix::removeImage(ImageMatrixItem *img)
{AA
	images->remove(img);
	img->Destroy();
}

void ImageMatrix::setSelection(ImageMatrixItem* img)
{AA
	void *node;
	void *obj = (void*)img;

	if(images == NULL) return;
	if(images->getSize() == 0) return;
	node = images->startIteration();
	while(node != NULL) {
		((ImageMatrixItem*)node)->setSelected((node == obj));
		node = images->next();
	}
}

void ImageMatrix::setSizes()
{AA
	Option *optcols = options.getOption("ImageMatrix.cols");
	Option *optrows = options.getOption("ImageMatrix.rows");
		
	int w, h;
	int imgh, imgw;
	
	if(images == NULL) return;
	if(images->getSize() == 0) return;

	// Calculate the columns, rows and view size
	int cols = optcols->getInt();
	int rows = optrows->getInt();

	int shrinkcols, shrinkrows;
	if(((rows - 1) * cols) > (rows * (cols - 1))) {
		shrinkrows = 1;
		shrinkcols = 0;
	} else {
		shrinkrows = 0;
		shrinkcols = 1;
	}
	while(images->getSize() <= (rows - shrinkrows) * (cols - shrinkcols)) {
		rows -= shrinkrows;
		cols -= shrinkcols;
		if(((rows - 1) * cols) > (rows * (cols - 1))) {
			shrinkrows = 1;
			shrinkcols = 0;
		} else {
			shrinkrows = 0;
			shrinkcols = 1;
		}
	}

	GetClientSize(&w, &h);
	imgh = h / rows;
	imgw = w / cols;

	//Resize and move the images
	ImageMatrixItem *img = (ImageMatrixItem*)images->startIteration();
	for(int y = 0; y < rows; y++) {
		for(int x = 0; x < cols; x++) {
			if(!img) return;
//			DEBUGPRINTPAR("+ [ImageMatrix::setSizes]: Resizing %s\n", img->getName()->c_str())
			img->SetSize(x * imgw + 1, y * imgh + 1, imgw - 2, imgh - 2);
			img = (ImageMatrixItem*)images->next();
		}
	}
}

void ImageMatrix::evtOnSize(wxSizeEvent& event) 
{AA
	setSizes();
	event.Skip();
}

void ImageMatrix::evtOnInsertImage(wxCommandEvent& event) 
{AA	
	addImage((ImageMatrixItem*)event.GetClientData());
}

void ImageMatrix::evtOnBtnCloseClick(wxCommandEvent& event) 
{AA
	wxWindow *wnd = (wxWindow*)event.GetEventObject();
	ImageMatrixItem *img = (ImageMatrixItem*)wnd->GetParent();

	if(img->isSelected()) {
		ImageMatrixItem *prev = NULL;
		ImageMatrixItem *item = (ImageMatrixItem*)images->startIteration();
		while(item) {
			if(item == img) {
				ImageMatrixItem *next = (ImageMatrixItem*)images->next();
				if(next) setSelection(next);
				else if(prev) setSelection(prev);
			}
			prev = item;
			item = (ImageMatrixItem*)images->next();
		}
	}
	
	removeImage(img);
	setSizes();
//	event.Skip();
}

void ImageMatrix::evtOnMouseClick(wxMouseEvent& event)
{AA
	if(!images) return;

	ImageMatrixItem* img = (ImageMatrixItem*)event.GetEventObject();

	// Make shure that an item actually was clicked, and not something void.
	ImageMatrixItem *node = (ImageMatrixItem*)images->startIteration();
	while(node) {
		if(node == img) {
			setSelection(img);
			return;
		}
		node = (ImageMatrixItem*)images->next();
	}
}
