/*
 * PC-Show - A crossplatform imageviewer, including integrated 
 * functionality with PC-Praxis.
 *
 * Copyright (C) 2004  Bent Bisballe Jensen
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "global.h"

#include "imagematrixitem.h"

//#include <wx/filename.h>
//#include <wx/dir.h>
//#include <wx/file.h>

BEGIN_EVENT_TABLE(ImageMatrixItem, wxPanel)
	EVT_LEFT_DOWN(ImageMatrixItem::evtOnMouseClick)
	EVT_LEFT_DCLICK(ImageMatrixItem::evtOnMouseClick)
	EVT_SIZE(ImageMatrixItem::evtOnSize)
	EVT_BUTTON(BTN_CLOSE, ImageMatrixItem::evtOnBtnCloseClick)
END_EVENT_TABLE()

ImageMatrixItem::ImageMatrixItem(Image* img) : wxPanel() 
{AA
	image = img;
	//image->connect();
	
	smallimg = NULL;
	bmpImage = NULL;
	
	//Initialize controls:
	lblName = NULL;
	txtComment = NULL;
	btnShowNotes = NULL;
	btnClose = NULL;
	imgImage = NULL;
}

ImageMatrixItem::~ImageMatrixItem() 
{AA
	if(smallimg) delete smallimg;
	if(bmpImage) delete bmpImage;
	//image->disconnect();
}

void ImageMatrixItem::setSelected(bool sel) 
{AA
	selected = sel;

	if(selected) SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_HIGHLIGHT));
	else SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_BTNFACE));

	Refresh(true);
}

bool ImageMatrixItem::isSelected()
{AA
	return selected;
}

void ImageMatrixItem::showFullscreen()
{AA
	if(!image) return;
	image->showFullscreen();
}

void ImageMatrixItem::evtOnMouseClick(wxMouseEvent& event)
{AA
	if(event.ButtonDClick()) showFullscreen();
	else {
		event.SetEventObject(this);
		GetParent()->GetEventHandler()->ProcessEvent(event);
	}
	event.Skip();
}

void ImageMatrixItem::evtOnSize(wxSizeEvent& event ) 
{AA
	if(event.GetEventObject() == this) {
		pack();
	}

	event.Skip();
}

void ImageMatrixItem::pack() 
{AA
	DEBUGPRINTPAR("->[ImageMatrixItem::pack] %s\n", getName()->c_str())
	
	if(!image) return;
		
	if(!smallimg) {
		image->connect();
		
		wxImage *wximg = image->getImage();
		
		// If wximg == NULL means that the imagefile was damaged.
		if(!wximg) return;//wximg = new wxImage(DEFAULT_ICON, wxBITMAP_TYPE_ANY, -1);

		int imgw = wximg->GetWidth();
		int imgh = wximg->GetHeight();
				
		double scalar = (double)320 / ((double)imgw + 0.0001);
		imgw = 320;
		imgh = (int)((double)imgh * scalar);

		smallimg = new wxImage(wximg->Scale(imgw, imgh));

		image->disconnect();
	}

	double fimgh = smallimg->GetHeight();
	double fimgw = smallimg->GetWidth();

	double scale;
	
	int h, w;
	double fh, fw;
	GetClientSize(&w, &h);
	h = h - (2 * (CTRLH + MARGIN));
	
	fh = (double)h + 0.0001;
	fw = (double)w + 0.0001;

	double scalew = fimgw / fw;
	double scaleh = fimgh / fh;

	if(scalew > scaleh) scale = scalew;
	else scale = scaleh;

	scale += 0.0001;
	
	int imgh = (int)(fimgh / scale) - (4 * MARGIN);
	int imgw = (int)(fimgw / scale) - (4 * MARGIN);
	
	if((imgw > 0) && (imgh > 0)) {
		if(bmpImage) delete bmpImage;
		bmpImage = new wxBitmap(smallimg->Scale(imgw, imgh));
		if(imgImage == NULL) {
			imgImage = new StaticImage(this, -1, *bmpImage);
		} else {
			imgImage->SetBitmap(*bmpImage);
		}
		imgImage->SetSize(wxSize(imgw, imgh));
		imgImage->Move(((w - imgw) / 2), ((h - imgh) / 2) - MARGIN);
	}

	//Set client width (without the margin)
	w -= MARGIN * 2;
	int ctrlx = MARGIN;
	int ctrlw = (int)(w * 0.6);
	
	if(lblName == NULL) {
		lblName = new wxStaticText(this, -1, getName()->c_str(), wxDefaultPosition,wxDefaultSize, wxTRANSPARENT_WINDOW);
	}
	lblName->SetSize(wxSize(ctrlw, CTRLH));
	lblName->Move(ctrlx, h - (2 * MARGIN));

	ctrlx += ctrlw + MARGIN;
	ctrlw = (int)(w * 0.4);

/*
	if(btnShowNotes == NULL) {
		btnShowNotes = new wxToggleButton(this, -1, "Show Notes");
	}
	btnShowNotes->SetSize(wxSize(ctrlw, CTRLH));
	btnShowNotes->Move(ctrlx, h - (2 * MARGIN));
*/
	ctrlx = MARGIN;
	ctrlw = (int)(w * 0.6);
/*
	if(txtComment == NULL) {
		txtComment = new wxTextCtrl(this, -1, "COMMENTS:Dette er en test");
	}
	txtComment->SetSize(wxSize(ctrlw, CTRLH));
	txtComment->Move(ctrlx, h + CTRLH);
*/
	ctrlx += ctrlw + MARGIN;
	ctrlw = (int)(w * 0.4);

	if(btnClose == NULL) {
		btnClose = new wxButton(this, BTN_CLOSE, "Luk");
	}
	btnClose->SetSize(wxSize(ctrlw, CTRLH));
	btnClose->Move(ctrlx, h + CTRLH);

	Show(true);
	Refresh(true);
}

void ImageMatrixItem::evtOnBtnCloseClick(wxCommandEvent& event)
{AA
	GetParent()->GetEventHandler()->ProcessEvent(event);
}

Image *ImageMatrixItem::getImage() 
{AA
	return image;
}

/*
wxImage *ImageMatrixItem::getImage() 
{AA
	if(!image) return NULL;
	return image->getImage(); 
}
*/

wxString *ImageMatrixItem::getName() 
{AA
	if(!image) return NULL;
	return image->getName(); 
}

wxString *ImageMatrixItem::getPath() 
{AA
	if(!image) return NULL;
	return image->getPath(); 
}

wxString *ImageMatrixItem::getDate() 
{AA
	if(!image) return NULL;
	return image->getDate(); 
}
