/*
 * PC-Show - A crossplatform imageviewer, including integrated 
 * functionality with PC-Praxis.
 *
 * Copyright (C) 2004  Bent Bisballe Jensen
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "linkedlist.h"

Link::Link(void* dta)
{AA
	data = dta;
	next = NULL;
	prev = NULL;
}

Link::~Link()
{AA
	if(next) next->prev = prev;
	if(prev) prev->next = next;
}

void* Link::getData()
{AA
	return data;
}

void Link::setData(void* dta)
{AA
	data = dta;
}

Link* Link::getNext()
{AA
	return next;
}

void Link::setNext(Link* nxt)
{AA
	next = nxt;
}

Link* Link::getPrev()
{AA
	return prev;
}

void Link::setPrev(Link* prv)
{AA
	prev = prv;
}


///////////////////////////
LinkedList::LinkedList()
{AA
	size = 0;
	current = NULL;
	first = NULL;
	last = NULL;
}

LinkedList::~LinkedList()
{AA
	while(getFirst() != NULL)
		remove(getFirst());
}

int LinkedList::getSize()
{AA
	return size;
}

void LinkedList::insert(void* obj)
{AA
	Link *lnk = new Link(obj);
	lnk->setNext(first);
	if(first) first->setPrev(lnk);
	first = lnk;
	if(!last) last = lnk;
	size++;
}

void LinkedList::remove(void* obj)
{AA
	Link *node = first;

	while(node) {
		if(node->getData() == obj) {
			if(first == node) first = node->getNext();
			if(last == node) last = node->getPrev();

			// Reset iterations
			current = NULL;

			delete node;
			size--;
			return;
		}
		node = node->getNext();
	}
}

void* LinkedList::getFirst()
{AA
	#ifdef __DEBUG
	printlist();
	#endif
	if(first == NULL) return NULL;
	return first->getData();
}

void* LinkedList::getLast()
{AA
	#ifdef __DEBUG
	printlist();
	#endif
	if(last == NULL) return NULL;
	return last->getData();
}

void* LinkedList::startIteration()
{AA
	#ifdef __DEBUG
	printlist();
	#endif
	current = first;
	if(!current) return NULL;
	return current->getData();
}

void* LinkedList::startIterationReversed()
{AA
	#ifdef __DEBUG
	printlist();
	#endif
	current = last;
	if(!current) return NULL;
	return current->getData();
}

void* LinkedList::next()
{AA
	if(!current) return NULL;
	current = current->getNext();
	if(!current) return NULL;
	return current->getData();
}

void* LinkedList::prev()
{AA
	if(!current) return NULL;
	current = current->getPrev();
	if(!current) return NULL;
	return current->getData();
}
#ifdef __DEBUG
void LinkedList::printlist()
{AA
	Link *node;

	fprintf(stderr, "==== LinkedList [%.9d] ====\n", (int)this);
	fprintf(stderr, "First: [%.9d]\n", (int)first);
	fprintf(stderr, "Last: [%.9d]\n", (int)last);
	
	node = first;
	while(node) {
		char name[256];
		
		if(node == first) sprintf(name, "first");
		else if(node == last) sprintf(name,  "last ");
		else sprintf(name, "node ");
		
		fprintf(stderr, "\t%.9d <- %s [%.9d] -> %.9d\n", 
			(int)node->getPrev(),
			name,
			(int)node,
			(int)node->getNext()
				);
		node = node->getNext();
	}

	fprintf(stderr, "========================\n");
}
#endif
