/*
 * PC-Show - A crossplatform imageviewer, including integrated 
 * functionality with PC-Praxis.
 *
 * Copyright (C) 2004  Bent Bisballe Jensen
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "global.h"

#include "mainwindow.h"

#include <wx/file.h>
#include <wx/cursor.h>

#include "fileselectoritem.h"
#include "imagematrixitem.h"
#include "options.h"
#include "image.h"
#include "pc-show.h"
#include "aboutwindow.h"

#include <wx/mimetype.h>
#include <wx/filesys.h>

// This is the global definition af the mainwindow
MainWindow* mainwindow;

BEGIN_EVENT_TABLE(MainWindow, wxFrame)
	// Menus
    EVT_MENU(MENU_SHOW_OPTIONS, MainWindow::menuShowOptions)
	EVT_MENU(MENU_SHOW_SELECTION, MainWindow::menuShowSelection)
	EVT_MENU(MENU_SHOW_FULLSCREEN, MainWindow::menuShowFullscreen)
    EVT_MENU(MENU_QUIT, MainWindow::menuQuit)
/*
	EVT_MENU(KEY_UP, MainWindow::keyUp)
    EVT_MENU(KEY_DOWN, MainWindow::keyDown)
    EVT_MENU(KEY_LEFT, MainWindow::keyLeft)
    EVT_MENU(KEY_RIGHT, MainWindow::keyRight)
*/
	EVT_MENU(MENU_FILE_DELETE, MainWindow::menuFileDelete)
	EVT_MENU(MENU_FILE_COPY, MainWindow::menuFileCopy)
	EVT_MENU(MENU_FILE_MOVE, MainWindow::menuFileMove)
	EVT_MENU(MENU_FILE_SAVEAS, MainWindow::menuFileSaveas)
	EVT_MENU(MENU_FILE_RENAME, MainWindow::menuFileRename)
	
	EVT_MENU(MENU_ABOUT, MainWindow::menuAbout)
	EVT_MENU(MENU_HELP, MainWindow::menuHelp)
	EVT_MENU(MENU_ONLINE_HELP, MainWindow::menuOnlineHelp)
	EVT_MENU(MENU_HOMEPAGE, MainWindow::menuHomepage)
	
	EVT_MENU(MENU_EXT_PROG_BASE + 0, MainWindow::menuRunExternal0)
	EVT_MENU(MENU_EXT_PROG_BASE + 1, MainWindow::menuRunExternal1)
	EVT_MENU(MENU_EXT_PROG_BASE + 2, MainWindow::menuRunExternal2)
	EVT_MENU(MENU_EXT_PROG_BASE + 3, MainWindow::menuRunExternal3)
	EVT_MENU(MENU_EXT_PROG_BASE + 4, MainWindow::menuRunExternal4)
	EVT_MENU(MENU_EXT_PROG_BASE + 5, MainWindow::menuRunExternal5)
	EVT_MENU(MENU_EXT_PROG_BASE + 6, MainWindow::menuRunExternal6)
	EVT_MENU(MENU_EXT_PROG_BASE + 7, MainWindow::menuRunExternal7)
	EVT_MENU(MENU_EXT_PROG_BASE + 8, MainWindow::menuRunExternal8)
	EVT_MENU(MENU_EXT_PROG_BASE + 9, MainWindow::menuRunExternal9)
	// Window events
	EVT_IDLE(MainWindow::evtIdle)
	EVT_SIZE(MainWindow::evtOnResize)
	EVT_MOVE(MainWindow::evtOnMove)

	EVT_ACTIVATE(MainWindow::evtOnAppActivate)

END_EVENT_TABLE()


MainWindow::MainWindow(): wxFrame(NULL, -1, "PC-Show")
{AA
	SetIcon(wxIcon("pc-show.ico", wxBITMAP_TYPE_ICO));
	
	Option *optheight = options.getOption("MainWindow.h");
	Option *optwidth = options.getOption("MainWindow.w");
	Option *optx = options.getOption("MainWindow.x");
	Option *opty = options.getOption("MainWindow.y");
	Option *optmax = options.getOption("MainWindow.maximized");
	Option *optextn = options.getOption("External.last");
	Option *extbin;
	Option *extname;
	Option *extkeys;
	
	watchfile = new Watchfile();
	filecache = new FileCache(watchfile);
	
	wxAcceleratorEntry entries[3];
	entries[0].Set(wxACCEL_CTRL,    (int) 'O',   MENU_SHOW_OPTIONS);
	entries[1].Set(wxACCEL_NORMAL,  (int) 'W',   MENU_SHOW_OPTIONS);
//	entries[2].Set(wxACCEL_NORMAL,  WXK_UP,		KEY_UP);
//	entries[3].Set(wxACCEL_NORMAL,  WXK_DOWN,	KEY_DOWN);
//	entries[4].Set(wxACCEL_NORMAL,  WXK_LEFT,	KEY_LEFT);
//	entries[5].Set(wxACCEL_NORMAL,  WXK_RIGHT,	KEY_RIGHT);
	entries[2].Set(wxACCEL_NORMAL,  (int) 'Z',	MENU_SHOW_FULLSCREEN);
	wxAcceleratorTable accel(3, entries);

	SetAcceleratorTable(accel);

	// Set the position and size of the mainwindow
	Move(wxPoint(optx->getInt(), opty->getInt()));
	SetSize(wxSize(optwidth->getInt(), optheight->getInt()));
//	fprintf(stderr, "Bool %d\n", optmax->getBool());
//	exit(1);
	Maximize(optmax->getBool() != false);

	CreateStatusBar(2);
	
	// Make the menubar
	wxMenu *fileMenu = new wxMenu();
	fileMenu->Append(MENU_SHOW_SELECTION, "Vis &valgte\tEnter", "Vis valgte.");
	fileMenu->AppendSeparator();
	fileMenu->Append(MENU_SHOW_FULLSCREEN, "Vis &fuldskaerm\tZ", "Vis valgte i fuldskaerm.");
	fileMenu->AppendSeparator();

//	fileMenu->Append(MENU_FILE_DELETE, "&Slet fil\tCtrl-D", "Slet den valgte fil.");
//	fileMenu->Append(MENU_FILE_COPY,   "&Kopier fil fra hoejre vindue\tCtrl-K", "Kopier den valgte fil.");
//	fileMenu->Append(MENU_FILE_MOVE,   "&Flyt fil\tCtrl-F", "Flyt den valgte fil.");
	fileMenu->Append(MENU_FILE_SAVEAS, "&Gem fil fra visningsvinduet\tCtrl-G", "Gem den valgte fil som.");
//	fileMenu->Append(MENU_FILE_RENAME, "&Omdoeb fil\tCtrl-O", "Omdoeb den valgte fil.");
/*
	fileMenu->Append(MENU_UP, "Up", "Up");
	fileMenu->Append(MENU_DOWN, "Down", "Down");
	fileMenu->Append(MENU_LEFT, "Left", "Left");
	fileMenu->Append(MENU_RIGHT, "Right", "Right");
*/
	fileMenu->AppendSeparator();
	fileMenu->Append(MENU_QUIT, "E&xit\tAlt-X", "Exit");
	
	wxMenu *toolsMenu = new wxMenu();
	toolsMenu->Append(MENU_SHOW_OPTIONS, "&Options\tCtrl-O", "Options");
	toolsMenu->AppendSeparator();
	for(int cnt = 0; cnt <= optextn->getInt(); cnt++) {
		char buf[64];
		sprintf(buf, "External.bin[%d]", cnt);
		extbin = options.getOption(buf);
		if(extbin->getFile()->CompareTo("") != 0) {
			sprintf(buf, "External.name[%d]", cnt);
			extname = options.getOption(buf);
	
			sprintf(buf, "External.keys[%d]", cnt);
			extkeys = options.getOption(buf);
	
			toolsMenu->Append(MENU_EXT_PROG_BASE + cnt, 
				extname->getString()->Append("\t").Append(extkeys->getString()->c_str()).c_str(), 
				extbin->getFile()->c_str()); 
		}
	}

	wxMenu *helpMenu = new wxMenu();
	helpMenu->Append(MENU_HELP, "&Help", "Help");
	helpMenu->Append(MENU_ONLINE_HELP, "&Online Help", "Online Help");
	helpMenu->AppendSeparator();
	helpMenu->Append(MENU_HOMEPAGE, "&Hjemmeside", "Product homepage");
	helpMenu->Append(MENU_ABOUT, "&About PC-Show", "About");
	

	wxMenuBar *menuBar = new wxMenuBar();
	menuBar->Append(fileMenu, "&Filer");
	menuBar->Append(toolsMenu, "&Tools");
	menuBar->Append(helpMenu, "&Help");

	SetMenuBar(menuBar);

	splitter = new MainWindowSplitter(this);
	imagematrix = new ImageMatrix(splitter);
	fileselector = new FileSelector(splitter);
	optionswindow = NULL;

	splitter->SplitVertically(fileselector, imagematrix, 100);

	SetStatusText("Velkommen til PC-Show", 1);
	
	imagematrix->SetAcceleratorTable(accel);
	fileselector->SetAcceleratorTable(accel);

	firstIdleEvent = NULL;
	lastIdleEvent = NULL;
}

MainWindow::~MainWindow() 
{AA	
	Option *optmax = options.getOption("MainWindow.maximized");
	optmax->setBool(IsMaximized());

	delete watchfile;
	delete filecache;
}

/**
 * refresh - Loads the watchfile, and checks for changes in the filelist,
 *  reloads if neccesary.
 */
void MainWindow::refresh()
{AA
	// Has the file changed?
	if(watchfile->read()) {
		DEBUGPRINT("+ [MainWindow::refresh] Watchfile changed.\n")
		// Delete the cache
		filecache->clear();
		// Clear the fileselctor list.
		fileselector->clear();
		// Remove all open files in imagematrix.
		imagematrix->clear();	

		// Stop reading more images....
//		clearIdleEventQueue();

		SetStatusText(*watchfile->getName(), 1);
	}
	
	// Verify the cache
	if(filecache->check()) {
		DEBUGPRINT("+ [MainWindow::refresh] Filecache changed.\n")
		// Stop reading more images....
//		clearIdleEventQueue();

		// Update filelist according to the file cache.
		fileselector->refresh(filecache);
	}
}

void MainWindow::evtOnMove(wxMoveEvent& event ) 
{AA
	Option *max = options.getOption("MainWindow.maximized");
	max->setBool(IsMaximized());

	Option *wndx = options.getOption("MainWindow.x");
	Option *wndy = options.getOption("MainWindow.y");
	int x, y;
	
	GetPosition(&x, &y);
	
	wndx->setInt(x);
	wndy->setInt(y);
	
	event.Skip();
}

void MainWindow::evtOnAppActivate(wxActivateEvent& event ) 
{AA
	if(event.GetActive()) {
		DEBUGPRINT("+ [MainWindow::evtOnAppActivate]\n")
		refresh();
	}
	event.Skip();
}

void MainWindow::evtOnResize(wxSizeEvent& event ) 
{AA
	Option *max = options.getOption("MainWindow.maximized");
	max->setBool(IsMaximized());
	
	Option *wndw = options.getOption("MainWindow.w");
	Option *wndh = options.getOption("MainWindow.h");
	int w, h;
	
	GetSize(&w, &h);
	
	wndw->setInt(w);
	wndh->setInt(h);

	event.Skip();
}


void MainWindow::showSelection() 
{AA
	long i;

	SetStatusText("Showing images.");

	if(fileselector->GetItemCount() == 0) return;
	i = fileselector->GetFirstSelected();
	if(i == -1) {
		FileSelectorItem *item = (FileSelectorItem*)fileselector->GetItemData(fileselector->GetFocusedItem());
		Image* img = item->getImage();

		insertIdleEvent(INSERT_IMAGE_INTO_IMAGEMATRIX, img);
	} else {
		while(i != -1) {
			FileSelectorItem *item = (FileSelectorItem*)fileselector->GetItemData(i);
			Image* img = item->getImage();

			insertIdleEvent(INSERT_IMAGE_INTO_IMAGEMATRIX, img);
				
			i = fileselector->GetNextSelected(i);
		}
		
		// Unselect all items after insertion.
		for(int cnt = 0; cnt < fileselector->GetItemCount(); cnt ++) fileselector->Select(cnt, false);
	}
	SetStatusText("Showing images...done!");
} 

void MainWindow::menuShowSelection(wxCommandEvent& WXUNUSED(event)) 
{AA
	showSelection();
}

void MainWindow::menuShowFullscreen(wxCommandEvent& WXUNUSED(event)) 
{AA
	DEBUGPRINT("ShowFullscreen\n")
	ImageMatrixItem *img = imagematrix->getSelected();
	if(!img) return;
	img->showFullscreen();
}

void MainWindow::menuShowOptions(wxCommandEvent& WXUNUSED(event)) 
{AA
	optionswindow = new OptionsWindow(this);
//	optionswindow->reset();
	optionswindow->ShowModal();
	optionswindow->Destroy();
}

void MainWindow::menuQuit(wxCommandEvent& WXUNUSED(event) ) 
{AA
    Close(true);
}
/*
void MainWindow::keyUp(wxCommandEvent& event) 
{AA
	fileselector->moveUp(event);
}

void MainWindow::keyDown(wxCommandEvent& event)
{AA
	fileselector->moveDown(event);
}

void MainWindow::keyLeft(wxCommandEvent& event)
{AA
	imagematrix->moveLeft(event);
}

void MainWindow::keyRight(wxCommandEvent& event)
{AA
	imagematrix->moveRight(event);
}
*/

void MainWindow::menuFileDelete(wxCommandEvent& WXUNUSED(event) ) 
{AA
	if(!imagematrix->getSelected()) return;
	ImageMatrixItem *img = imagematrix->getSelected();
	
	wxMessageDialog *dlg = new wxMessageDialog(this, "Ok at slette filen?", "Slet?", wxOK | wxCANCEL);
	if(dlg->ShowModal() == wxID_OK) {	
		wxRemoveFile(*img->getPath());
//		fileselector->refresh();
	}
}

void MainWindow::menuFileCopy(wxCommandEvent& WXUNUSED(event) ) 
{AA
	ImageMatrixItem *img = imagematrix->getSelected();
	if(!img) return;

	wxDirDialog *dlg = new wxDirDialog(this);
	dlg->SetPath(*img->getPath());
	if(dlg->ShowModal() == wxID_OK) {	
		wxString dst = dlg->GetPath();
		dst.Append("/");
		dst.Append(img->getName()->c_str());
		wxCopyFile(*img->getPath(), dst);
//		fileselector->refresh();
	}
}

void MainWindow::menuFileMove(wxCommandEvent& WXUNUSED(event) ) 
{AA
	ImageMatrixItem *img = imagematrix->getSelected();
	if(!img) return;
	
	wxDirDialog *dlg = new wxDirDialog(this);
	dlg->SetPath(*img->getPath());
	if(dlg->ShowModal() == wxID_OK) {
		wxString dst = dlg->GetPath();
		dst.Append("/");
		dst.Append(img->getName()->c_str());
		wxRenameFile(*img->getPath(), dst);
//		fileselector->refresh();
	}
}

void MainWindow::menuFileSaveas(wxCommandEvent& WXUNUSED(event) ) 
{AA
	ImageMatrixItem *img = imagematrix->getSelected();
	if(!img) return;

	Option *opt = options.getOption("SaveAs.defaultpath");

	wxFileDialog *dlg = new wxFileDialog(this);
	dlg->SetDirectory(*opt->getDir());
	dlg->SetFilename(img->getPath()->AfterLast('/'));
//	dlg->SetPath(*opt->getDir());

//	dlg->SetStyle(wxCHANGE_DIR);
	
//	dlg->SetPath(*img->getPath());
	if(dlg->ShowModal() == wxID_OK) {
		int type = 0;

		wxString *ext = new wxString(dlg->GetPath().AfterLast('.').Lower());

		if(ext->CompareTo("bmp") == 0) 	type = wxBITMAP_TYPE_BMP;
		else if(ext->CompareTo("jpg") == 0)	type = wxBITMAP_TYPE_JPEG;
		else if(ext->CompareTo("jpeg") == 0)	type = wxBITMAP_TYPE_JPEG;
		else if(ext->CompareTo("png") == 0)	type = wxBITMAP_TYPE_PNG;
		else if(ext->CompareTo("pcx") == 0)	type = wxBITMAP_TYPE_PCX;
		else if(ext->CompareTo("pnm") == 0)	type = wxBITMAP_TYPE_PNM;
//		else if(ext->CompareTo("tif") == 0)	type = wxBITMAP_TYPE_TIF;
//		else if(ext->CompareTo("tiff") == 0)	type = wxBITMAP_TYPE_TIF;
		else if(ext->CompareTo("xpm") == 0)	type = wxBITMAP_TYPE_XPM;
		else {
			wxMessageDialog *msgdlg = new wxMessageDialog(this, "Ukendt fil type!", "Kan ikke gemme fil...", wxOK);
			msgdlg->ShowModal();
		}
		
		Image *image = img->getImage();
		
		// If image == NULL it means that the imagefile is corrupted.
		if(!image) return;
		
		if(type) {
			if(wxFileExists(dlg->GetPath())) {
				wxMessageDialog *msgdlg = new wxMessageDialog(this, "Filen findes allerede, skal den overskrives?", "Kan ikke gemme fil...", wxYES_NO);
				if(msgdlg->ShowModal() == wxID_YES) {
					DEBUGPRINT("I took yes for an answer`.!\n")
					if(wxRemoveFile(dlg->GetPath())) {
						DEBUGPRINT("Could delete!!!\n")
						image->connect();
						image->getImage()->SaveFile(dlg->GetPath(), type);
						image->disconnect();
					} else {
						DEBUGPRINT("Could not delete!!!\n")
						wxMessageDialog *msgdlg = new wxMessageDialog(this, "Filen kunne ikke overskrives. Check filsystemsrettighederne.", "Kan ikke gemme fil...", wxOK);
						msgdlg->ShowModal();
					}
				} else DEBUGPRINT("I took no for an answer`.!\n")
			} else {
				DEBUGPRINT("File wasn't here already... lets write!\n")
				image->connect();
				image->getImage()->SaveFile(dlg->GetPath(), type);
				image->disconnect();
			}
		}
	}
	// Make sure the new file will be shown in the filelist.
	refresh();
}

void MainWindow::menuFileRename(wxCommandEvent& WXUNUSED(event) ) 
{AA
	ImageMatrixItem *img = imagematrix->getSelected();
	if(!img) return;

	wxTextEntryDialog *dlg = new wxTextEntryDialog(this, "Omdoeb filen");
	dlg->SetValue(*img->getName());
	if(dlg->ShowModal() == wxID_OK) {
		wxString dst = img->getPath()->BeforeLast('/');
		dst.Append("/");
		dst.Append(dlg->GetValue().c_str());
		wxRenameFile(*img->getPath(), dst);
//		fileselector->refresh();
	}
	
}

void MainWindow::menuRunExternal0(wxCommandEvent& WXUNUSED(event) )
{AA
	Option *optbin = options.getOption("External.bin[0]");
	DEBUGPRINTPAR("[MainWindow::menuRunExternal0] Executing %s\n",optbin->getFile()->c_str())
	wxExecute(optbin->getFile()->c_str());
}

void MainWindow::menuRunExternal1(wxCommandEvent& WXUNUSED(event) )
{AA
	Option *optbin = options.getOption("External.bin[1]");
	DEBUGPRINTPAR("[MainWindow::menuRunExternal1] Executing %s\n",optbin->getFile()->c_str())
	wxExecute(optbin->getFile()->c_str());
}

void MainWindow::menuRunExternal2(wxCommandEvent& WXUNUSED(event) )
{AA
	Option *optbin = options.getOption("External.bin[2]");
	DEBUGPRINTPAR("[MainWindow::menuRunExternal2] Executing %s\n",optbin->getFile()->c_str())
	wxExecute(optbin->getFile()->c_str());
}

void MainWindow::menuRunExternal3(wxCommandEvent& WXUNUSED(event) )
{AA
	Option *optbin = options.getOption("External.bin[3]");
	DEBUGPRINTPAR("[MainWindow::menuRunExternal3] Executing %s\n",optbin->getFile()->c_str())
	wxExecute(optbin->getFile()->c_str());
}

void MainWindow::menuRunExternal4(wxCommandEvent& WXUNUSED(event) )
{AA
	Option *optbin = options.getOption("External.bin[4]");
	DEBUGPRINTPAR("[MainWindow::menuRunExternal4] Executing %s\n",optbin->getFile()->c_str())
	wxExecute(optbin->getFile()->c_str());
}

void MainWindow::menuRunExternal5(wxCommandEvent& WXUNUSED(event) )
{AA
	Option *optbin = options.getOption("External.bin[5]");
	DEBUGPRINTPAR("[MainWindow::menuRunExternal5] Executing %s\n",optbin->getFile()->c_str())
	wxExecute(optbin->getFile()->c_str());
}

void MainWindow::menuRunExternal6(wxCommandEvent& WXUNUSED(event) )
{AA
	Option *optbin = options.getOption("External.bin[6]");
	DEBUGPRINTPAR("[MainWindow::menuRunExternal6] Executing %s\n",optbin->getFile()->c_str())
	wxExecute(optbin->getFile()->c_str());
}

void MainWindow::menuRunExternal7(wxCommandEvent& WXUNUSED(event) )
{AA
	Option *optbin = options.getOption("External.bin[7]");
	DEBUGPRINTPAR("[MainWindow::menuRunExternal7] Executing %s\n",optbin->getFile()->c_str())
	wxExecute(optbin->getFile()->c_str());
}

void MainWindow::menuRunExternal8(wxCommandEvent& WXUNUSED(event) )
{AA
	Option *optbin = options.getOption("External.bin[8]");
	DEBUGPRINTPAR("[MainWindow::menuRunExternal8] Executing %s\n",optbin->getFile()->c_str())
	wxExecute(optbin->getFile()->c_str());
}

void MainWindow::menuRunExternal9(wxCommandEvent& WXUNUSED(event) )
{AA
	Option *optbin = options.getOption("External.bin[9]");
	DEBUGPRINTPAR("[MainWindow::menuRunExternal9] Executing %s\n",optbin->getFile()->c_str())
	wxExecute(optbin->getFile()->c_str());
}

void MainWindow::menuAbout(wxCommandEvent& WXUNUSED(event) )
{AA
	AboutWindow *about = new AboutWindow(this);
	about->ShowModal();
}

void MainWindow::menuOnlineHelp(wxCommandEvent& WXUNUSED(event) )
{AA	
	wxString url("http://www.aasimon.org/pc-show/?page=help");
	wxFileType *ft = wxTheMimeTypesManager->GetFileTypeFromExtension(wxT("html"));                   
    if ( !ft )                                                                  
    {                                
        wxLogError(_T("Impossible to determine the file type for extension html. Please edit your MIME types."));                                          
        return ;                                                                
    }                                                                           
                                                                                
    wxString cmd;                                                               
    bool ok = ft->GetOpenCommand(&cmd, wxFileType::MessageParameters(url, _T("")));   
    delete ft;                                                                  
                                                                                
    if (!ok)                                                                    
    {                                                                           
        // TODO: some kind of configuration dialog here.                        
        wxMessageBox(_("Could not determine the command for running the browser."),                                                                     
                   wxT("Browsing problem"), wxOK|wxICON_EXCLAMATION);           
        return ;                                                                
    }                                                                           
                                                                                                                                                                
    ok = (wxExecute(cmd, FALSE) != 0);                                          

}

void MainWindow::menuHomepage(wxCommandEvent& WXUNUSED(event) )
{AA	
	wxString url("http://www.aasimon.org/pc-show");
	wxFileType *ft = wxTheMimeTypesManager->GetFileTypeFromExtension(wxT("html"));                   
    if ( !ft )                                                                  
    {                                
        wxLogError(_T("Impossible to determine the file type for extension html. Please edit your MIME types."));                                          
        return ;                                                                
    }                                                                           
                                                                                
    wxString cmd;                                                               
    bool ok = ft->GetOpenCommand(&cmd, wxFileType::MessageParameters(url, _T("")));   
    delete ft;                                                                  
                                                                                
    if (!ok)                                                                    
    {                                                                           
        // TODO: some kind of configuration dialog here.                        
        wxMessageBox(_("Could not determine the command for running the browser."),                                                                     
                   wxT("Browsing problem"), wxOK|wxICON_EXCLAMATION);           
        return ;                                                                
    }                                                                           
                                                                                                                                                                
    ok = (wxExecute(cmd, FALSE) != 0);                                          

}

void MainWindow::menuHelp(wxCommandEvent& WXUNUSED(event) )
{AA
	wxString url = wxFileSystem::FileNameToURL(wxFileName("help.html"));
	
	wxFileType *ft = wxTheMimeTypesManager->GetFileTypeFromExtension(wxT("html"));                   
    if ( !ft )                                                                  
    {                                
        wxLogError(_T("Impossible to determine the file type for extension html. Please edit your MIME types."));                                          
        return ;                                                                
    }                                                                           
                                                                                
    wxString cmd;                                                               
    bool ok = ft->GetOpenCommand(&cmd, wxFileType::MessageParameters(url, _T("")));   
    delete ft;                                                                  
                                                                                
    if (!ok)                                                                    
    {                                                                           
        // TODO: some kind of configuration dialog here.                        
        wxMessageBox(_("Could not determine the command for running the browser."),                                                                     
                   wxT("Browsing problem"), wxOK|wxICON_EXCLAMATION);           
        return ;                                                                
    }                                                                           
                                                                                
    if (cmd.Find(wxT("http://")) != -1)                                         
        cmd.Replace(wxT("file://"), wxT(""));                    
                                                                                
    ok = (wxExecute(cmd, FALSE) != 0);                                          

}

// Idle Event Functions
void MainWindow::clearIdleEventQueue()
{
	idleEvent *evt = firstIdleEvent;
	idleEvent *deadEvent;
	
	while(evt) {
		deadEvent = evt;
		evt = evt->next;
		free(deadEvent);
	}
	firstIdleEvent = NULL;
	lastIdleEvent = NULL;
}

void MainWindow::insertIdleEvent(int type, void* data)
{AA
	idleEvent *evt;
	
	evt = (idleEvent*) malloc(sizeof(idleEvent));
	evt->type = type;
	evt->data = data;
	evt->next = NULL;
	if(lastIdleEvent) lastIdleEvent->next = evt;
	lastIdleEvent = evt;
	if(firstIdleEvent == NULL) firstIdleEvent = evt;
}

void MainWindow::evtIdle(wxIdleEvent& event)
{AA
	idleEvent *deadEvent;
	if(firstIdleEvent != NULL) {
		
		DEBUGPRINTPAR("IdleEvent: %d", firstIdleEvent->type)
		
		switch(firstIdleEvent->type) {
			case INSERT_IMAGE_INTO_IMAGEMATRIX:
				imagematrix->addImage(new ImageMatrixItem((Image*)firstIdleEvent->data));
				break;

			case LOAD_THUMBNAIL:
				fileselector->loadNextThumbnail((FileSelectorItem*)firstIdleEvent->data);
				break;
		}
		// if the first element all of a sudden is NULL, it means that the
		// idleEventList has been forced empty (probably by a corrupt image)
		if(firstIdleEvent == NULL) return;

		deadEvent = firstIdleEvent;
		firstIdleEvent = firstIdleEvent->next;
		if(firstIdleEvent == NULL) lastIdleEvent = NULL;
		free(deadEvent);
	}
	event.Skip();	
}
