/*
 * PC-Show - A crossplatform imageviewer, including integrated 
 * functionality with PC-Praxis.
 *
 * Copyright (C) 2004  Bent Bisballe Jensen
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __MAINWINDOW_H__
#define __MAINWINDOW_H__

#include <wx/event.h>
#include <wx/frame.h>

#include "mainwindowsplitter.h"
#include "fileselector.h"
#include "imagematrix.h"
#include "optionswindow.h"
#include "watchfile.h"
#include "filecache.h"

#include "events.h"

// Idle handler event types
enum
{
	INSERT_IMAGE_INTO_IMAGEMATRIX,
	LOAD_THUMBNAIL
};

typedef struct idleEvent {
	struct idleEvent *next;
	int type;
	void* data;
} idleEvent;



class MainWindow: public wxFrame
{
public:
	MainWindow();
	~MainWindow();

	void showSelection();	// Show the selected files.
	void refresh();

	void insertIdleEvent(int type, void* data);
	void clearIdleEventQueue();

	FileSelector		*fileselector;
	ImageMatrix			*imagematrix;
private:
	Watchfile *watchfile;
	FileCache *filecache;

	// Menu commands
	void menuShowOptions(wxCommandEvent& event);
	void menuShowSelection(wxCommandEvent& event);
	void menuShowFullscreen(wxCommandEvent& WXUNUSED(event));
	void menuQuit(wxCommandEvent& event);

	void menuAbout(wxCommandEvent& event);
	void menuHelp(wxCommandEvent& event);
	void menuOnlineHelp(wxCommandEvent& event);
	void menuHomepage(wxCommandEvent& event);

	void menuFileDelete(wxCommandEvent& WXUNUSED(event) );
	void menuFileCopy(wxCommandEvent& WXUNUSED(event) );
	void menuFileMove(wxCommandEvent& WXUNUSED(event) );
	void menuFileSaveas(wxCommandEvent& WXUNUSED(event) );
	void menuFileRename(wxCommandEvent& WXUNUSED(event) );

	void menuRunExternal0(wxCommandEvent& WXUNUSED(event));
	void menuRunExternal1(wxCommandEvent& WXUNUSED(event));
	void menuRunExternal2(wxCommandEvent& WXUNUSED(event));
	void menuRunExternal3(wxCommandEvent& WXUNUSED(event));
	void menuRunExternal4(wxCommandEvent& WXUNUSED(event));
	void menuRunExternal5(wxCommandEvent& WXUNUSED(event));
	void menuRunExternal6(wxCommandEvent& WXUNUSED(event));
	void menuRunExternal7(wxCommandEvent& WXUNUSED(event));
	void menuRunExternal8(wxCommandEvent& WXUNUSED(event));
	void menuRunExternal9(wxCommandEvent& WXUNUSED(event));

	//Idle event handler funcs
	idleEvent *firstIdleEvent;
	idleEvent *lastIdleEvent;

	// Event methods
	void evtOnResize(wxSizeEvent& event);
	void evtOnMove(wxMoveEvent& event);
	void evtActivated(wxActivateEvent& WXUNUSED(event) );
	void evtIdle(wxIdleEvent& event);
	void evtOnAppActivate(wxActivateEvent& event );
//	void evtOnGotFocus(wxFocusEvent& event );
	// Windows
	OptionsWindow		*optionswindow;
	MainWindowSplitter	*splitter;

	DECLARE_EVENT_TABLE()
};

extern MainWindow* mainwindow;

#endif
