/*
 * PC-Show - A crossplatform imageviewer, including integrated 
 * functionality with PC-Praxis.
 *
 * Copyright (C) 2004  Bent Bisballe Jensen
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "options.h"

// This is the global definition af the options object.
Options options;

wxString getConfDir() 
{AA
#ifdef __WXMSW__
	// Use fixed path on MSW
	return wxString("c:/x/.pc-show");
#else
	// Use homedir on *nix
	return wxGetHomeDir().Append("/.pc-show");
#endif
}

Options::Options() 
{AA
	Option *nulloption = new Option("NULL", "NULL");
	nulloption->setName("NULL");
	nulloption->setString("NULL");
	nulloption->setBool(true);
	nulloption->setInt(42);
	nulloption->setNext(NULL);
	
	first = nulloption;
	load();
}

Options::~Options() 
{AA
	save();
}

void Options::insertOption(Option* opt)
{AA
	DEBUGPRINTPAR("+ [Options::insertOption] Inserting [%s]\n", opt->getName()->c_str())
	Option *tstopt = first;
	while(tstopt) {
		if(tstopt->getName()->CompareTo(opt->getName()->c_str()) == 0) {
			DEBUGPRINTPAR("+ [Options::insertOption] Duplicate found [%s]\n", opt->getName()->c_str())
			tstopt->setName(*opt->getName());
			if(opt->getType() == TYPE_STR) tstopt->setString(*opt->getString());
			if(opt->getType() == TYPE_FILE) tstopt->setFile(*opt->getFile());
			if(opt->getType() == TYPE_DIR) tstopt->setDir(*opt->getDir());
			if(opt->getType() == TYPE_INT) tstopt->setInt(opt->getInt());
			if(opt->getType() == TYPE_BOOL) tstopt->setBool(opt->getBool());
			if(opt->getType() == TYPE_NONE) fprintf(stderr, "+ [Options::insertOption] A NULL type option detected in [%s]!\n", opt->getName());
			return;
		}
		tstopt = tstopt->getNext();
	}
	opt->setNext(first);
	first = opt;
}

Option *Options::getOption(const char *name)
{AA
	wxString *sname = new wxString(name); 
	Option *opt = first;
	while(opt) {
		if(sname->CompareTo("NULL") == 0) {
			fprintf(stderr, "+ [Options::getOptions] The NULLOPTION returned on request for [%s]\n", name);
			return opt;
		}
		if(sname->CompareTo(opt->getName()->c_str()) == 0) return opt;
		opt = opt->getNext();
	}
	
	// The Option was not found... creating it
	opt = new Option(name);
	DEBUGPRINTPAR("+ [Options::getOption] Option [%s] not found... creating!\n", opt->getName()->c_str())
	insertOption(opt);
	return opt;
}

bool Options::save() 
{AA
	wxString path = getConfDir();
	if(!wxDir::Exists(path.c_str())) {
		wxMkdir(path.c_str());
	}

	wxString fname = getConfDir().Append("/pc-show.conf");
	return saveFile((char*)fname.c_str());
}

bool Options::saveFile(char *fname) 
{AA
	wxLogNull noLogging;
	wxFile *file = new wxFile(fname, wxFile::write);
	if(!file->IsOpened()) {
		fprintf(stderr, "+ [Options::save] Unable to open the conf file for writing [%s]\n", fname);
		return false;
	}

	// Write the header (defined in options.h)
	file->Write(CONF_HEADER, strlen(CONF_HEADER));

	Option *opt = first;
	while(opt) {
		char buf[1024];
		if(opt->getToBeSaved()) {
			if(opt->getName()->CompareTo("NULL") == 0) break;
			
			// Type
			sprintf(buf, "%d", opt->getType());
			file->Write((void*)buf, strlen(buf));

			// To be saved
			if(opt->getToBeSaved()) file->Write("1", 1);
			else file->Write("0", 1);

			// Show in options
			if(opt->getShowInOptions()) file->Write("1", 1);
			else file->Write("0", 1);

			// Name
			file->Write(*opt->getName());

			file->Write("=", 1);

			// Value
			switch(opt->getType())  {
				case TYPE_FILE:
					file->Write(*opt->getFile());
					break;
				case TYPE_DIR:
					file->Write(*opt->getDir());
					break;
				case TYPE_STR:
					file->Write(*opt->getString());
					break;
				case TYPE_INT:
					sprintf(buf, "%d", opt->getInt());
					file->Write((void*)buf, strlen(buf));
					break;
				case TYPE_BOOL:
					sprintf(buf, "%d", opt->getBool());
					file->Write((void*)buf, strlen(buf));
					break;
				case TYPE_NONE:
					fprintf(stderr, "+ [Options::save] \Error: Trying to write an uninitialized option [%s]\n", opt->getName()->c_str());
					break;
			}
			file->Write("\n",1);	// Newline
		}		
		opt = opt->getNext();
	}
	file->Close();
	return true;
}

bool Options::load() 
{AA
	// Set vars
	wxString globalinifile("pc-show.conf");
	wxString inifile = getConfDir().Append("/pc-show.conf");
	wxString inipath = getConfDir();
	// Indlæs global defaults
	loadFile((char*)globalinifile.c_str());
	
	// TODO: Opret folder hvis den ikke findes

	// TODO: Opret conf fil hvis den ikke findes, fyld den ud med default værdierne 
		//	(evt. bare et kald til save())
	// else 
	// Indlæs user conf
	loadFile((char*)inifile.c_str());
	
	// Override the Paths.folder attribute
	insertOption(new Option("Paths.folder", ""));
	return true;
}

bool Options::loadFile(char *fname) 
{AA
	wxLogNull noLogging;
	Option *opt;

	wxFile *file = new wxFile(fname, wxFile::read);
	if(!file->IsOpened()) {
		fprintf(stderr, "+ [Options::load] Unable to open the conf file for reading [%s]\n", fname);
		return false;
	}
	
	char *buf;

	int bufsize = file->SeekEnd() + 1;
	file->Seek(0);

	buf = (char*)calloc(1, bufsize);
	file->Read(buf, bufsize);

	wxString buffer = wxString(buf);
	buffer.Replace("\r", "");
	
	wxStringTokenizer *tkz = new wxStringTokenizer(buffer.c_str(), "\n");
	while ( tkz->HasMoreTokens() )
	{
		wxString token = tkz->GetNextToken();
//		DEBUGPRINTPAR("+ [Options::load] Read: [%s]\n", token.c_str())
		
		token = token.Strip(wxString::both);
		token = token.BeforeFirst('#');
//		DEBUGPRINTPAR("+ [Options::load] Poststrip: [%s]\n", token.c_str())
		
		if(token.Length() > 0) {
//			DEBUGPRINT("+ [Options::load] Processing...\n")
			wxString typestr = token.Mid(0, 1);
			wxString savestr = token.Mid(1, 1);
			wxString showstr = token.Mid(2, 1);
			wxString name = (token.Mid(3)).BeforeFirst('=');
			wxString valstr = (token.Mid(3)).AfterFirst('=');
	
//			DEBUGPRINTPAR("+ [Options::load] \tTypestr: [%s]\n", typestr.c_str())
//			DEBUGPRINTPAR("+ [Options::load] \tShowstr: [%s]\n", showstr.c_str())
//			DEBUGPRINTPAR("+ [Options::load] \tSavestr: [%s]\n", savestr.c_str())
			DEBUGPRINTPARPAR("+ [Options::load] [%s]=[%s]\n", name.c_str(), valstr.c_str())
//			DEBUGPRINTPAR("Valstr: [%s]\n", valstr.c_str())
			
			opt = new Option(name.c_str());
			long int li;
			long int val;
			typestr.ToLong(&li);
			switch(li) {
				case TYPE_FILE:
					opt->setFile(valstr);
					break;
	
				case TYPE_DIR:
					opt->setDir(valstr);
					break;
	
				case TYPE_STR:
					opt->setString(valstr);
					break;
				
				case TYPE_INT:
					valstr.ToLong(&val);
					opt->setInt(val);
					break;
				
				case TYPE_BOOL:
					valstr.ToLong(&val);
					opt->setBool((bool)val);
					break;
				
				default:
					fprintf(stderr, "+ [Options::load] Unknown type [%s]\n", typestr.c_str());
					break;
			}
			savestr.ToLong(&li);
			opt->setToBeSaved((bool)li);
			showstr.ToLong(&li);
			opt->setShowInOptions((bool)li);
			insertOption(opt);
		}
	}
	file->Close();
	free(buf);
	return true;
/*	
	insertOption(new Option("MainWindow.x", 0));
	insertOption(new Option("MainWindow.y", 0));
	insertOption(new Option("MainWindow.w", 1024));
	insertOption(new Option("MainWindow.h", 768));
	insertOption(new Option("MainWindow.maximized", false));
//	insertOption(new Option("Sash.position", 220));

	wxString path = wxGetHomeDir().Append("/images");
	insertOption(new Option("Paths.folder", (char *)path.c_str()));
	wxString watch = wxGetHomeDir().Append("/pc-show.watch");
	insertOption(new Option("Paths.watchfile", (char *)watch.c_str()));

	insertOption(new Option("ImageMatrix.rows", 3));
	insertOption(new Option("ImageMatrix.cols", 3));
*/
}

Option *Options::getFirst()
{AA
	return first;
}


//////////////OPTION

Option::Option()
{AA
	next = NULL;
	type = TYPE_NONE;
	tobesaved = false;
	showinoptions = false;
}

Option::Option(const char *gname)
{AA
	name = new wxString(gname);
	next = NULL;
	strval = NULL;
	intval = 0;
	boolval = 0;
	type = TYPE_NONE;
	tobesaved = false;
	showinoptions = false;
}

Option::Option(const char *gname, char *gstrval)
{AA
	name = new wxString(gname);
	strval = new wxString(gstrval);	
	next = NULL;
	type = TYPE_STR;
	tobesaved = false;
	showinoptions = false;
}

Option::Option(const char *gname, int gintval)
{AA
	name = new wxString(gname);
	intval = gintval;
	next = NULL;
	type = TYPE_INT;
	tobesaved = false;
	showinoptions = false;
}

Option::Option(const char *gname, bool gboolval)
{AA
	name = new wxString(gname);
	boolval = gboolval;
	next = NULL;
	type = TYPE_BOOL;
	tobesaved = false;
	showinoptions = false;
}

wxString *Option::getName()
{AA
	return name;
}

void Option::setName(const char* gname)
{AA
	delete name;
	name = new wxString(gname);
}

int Option::getInt() 
{AA
	return intval;
}

void Option::setInt(int gintval)
{AA
	intval = gintval;
	type = TYPE_INT;
}

wxString *Option::getString()
{AA
	if(!strval) strval = new wxString("");
	return strval;
}

void Option::setString(wxString& gstrval)
{AA
	strval = new wxString(gstrval);
	type = TYPE_STR;
}

void Option::setString(char* gstrval)
{AA
	delete strval;
	strval = new wxString(gstrval);
	type = TYPE_STR;
}

wxString *Option::getDir()
{AA
	if(!strval) strval = new wxString("");
	return strval;
}

void Option::setDir(wxString& gstrval)
{AA
	strval = new wxString(gstrval);
	type = TYPE_DIR;
}

void Option::setDir(char* gstrval)
{AA
	delete strval;
	strval = new wxString(gstrval);
	type = TYPE_DIR;
}

wxString *Option::getFile()
{AA
	if(!strval) strval = new wxString("");
	return strval;
}

void Option::setFile(wxString& gstrval)
{AA
	strval = new wxString(gstrval);
	type = TYPE_FILE;
}

void Option::setFile(char* gstrval)
{AA
	delete strval;
	strval = new wxString(gstrval);
	type = TYPE_FILE;
}

bool Option::getBool()
{AA
	return boolval;
}

void Option::setBool(bool gboolval)
{AA
	boolval = gboolval;
	type = TYPE_BOOL;
}

int Option::getType()
{AA
	return type;
}

Option *Option::getNext()
{AA
	return next;
}

void Option::setNext(Option *gnext)
{AA
	next = gnext;
}

bool Option::getShowInOptions()
{AA
	return showinoptions;
}

void Option::setShowInOptions(bool show)
{AA
	showinoptions = show;
}

bool Option::getToBeSaved()
{AA
	return tobesaved;
}

void Option::setToBeSaved(bool saved)
{AA
	tobesaved = saved;
}
