/*
 * PC-Show - A crossplatform imageviewer, including integrated 
 * functionality with PC-Praxis.
 *
 * Copyright (C) 2004  Bent Bisballe Jensen
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __OPTIONS_H__
#define __OPTIONS_H__

#include "global.h"

#include <wx/file.h>
#include <wx/tokenzr.h>
#include <wx/dir.h>

enum {
	TYPE_NONE = 0,
	TYPE_STR  = 1,
	TYPE_INT  = 2,
	TYPE_BOOL = 3,
	TYPE_FILE = 4,
	TYPE_DIR  = 5
};

#define CONF_HEADER "######################################\n\
# PC-Show configurationfile\n\
######################################\n\
\n\
######################################\n\
# ----- Setting format\n\
# [n1][n2][n3][string1]=[string2]\n\
# ----- Explanations\n\
# [n1] Setting type: 0(none)\n\
#                    1(string)\n\
#                    2(signed integer)\n\
#                    3(boolean)\n\
#                    4(filename)\n\
#                    5(path)\n\
# [n2] Save in user conf file? \n\
#                (0 for no, 1 for yes)\n\
# [n3] Show in options window?\n\
#                (0 for no, 1 for yes)\n\
# [string1] Option name (no spaces)\n\
# [string2] Option value\n\
######################################\n\n"

class Option {
public:
	Option();
	Option(const char *gname);
	Option(const char *gname, char* gstrval);
	Option(const char *gname, int gintval);
	Option(const char *gname, bool gboolval);

	wxString *getName();
	void setName(const char* gname);
	int getInt();
	void setInt(int gintval);

	wxString *getFile();
	void setFile(char* gstrval);
	void setFile(wxString& gstrval);

	wxString *getDir();
	void setDir(char* gstrval);
	void setDir(wxString& gstrval);

	wxString *getString();
	void setString(char* gstrval);
	void setString(wxString& gstrval);

	bool getBool();
	void setBool(bool gboolval);

	bool getShowInOptions();
	void setShowInOptions(bool show);
	bool getToBeSaved();
	void setToBeSaved(bool saved);

	int getType();
	Option *getNext();
	void setNext(Option *gnext);
	
private:
	int type;
	wxString *name;
	wxString *strval;
	int intval;
	bool boolval;
	bool showinoptions;
	bool tobesaved;
	struct Option *next;
};


class Options {
public:
	Options();
	~Options();
	bool save();
	bool load();
	Option *getFirst();
	Option *getOption(const char *name);
	void insertOption(Option* opt);
private:
	bool saveFile(char *fname);
	bool loadFile(char *fname);
	Option *first;
};

extern Options options;

#endif
