/*
 * PC-Show - A crossplatform imageviewer, including integrated 
 * functionality with PC-Praxis.
 *
 * Copyright (C) 2004  Bent Bisballe Jensen
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "global.h"

#include "optionswindow.h"

#include <wx/button.h>
#include <wx/textctrl.h>
#include <wx/stattext.h>

BEGIN_EVENT_TABLE(OptionsWindow, wxDialog)
	EVT_BUTTON(BTN_BROWSEDIR, OptionsWindow::evtOnBrowseDir)
	EVT_BUTTON(BTN_BROWSEFILE, OptionsWindow::evtOnBrowseFile)
	EVT_BUTTON(BTN_OK, OptionsWindow::evtOnOk)
	EVT_BUTTON(BTN_CANCEL, OptionsWindow::evtOnCancel)
END_EVENT_TABLE()

OptionsWindow::OptionsWindow(wxWindow *parent):	wxDialog(parent, -1, "Options")
{AA
	Option *opt;
	int y = 5;
	wxTextCtrl *ctrl;
	children = new wxList();
	
	// Insert the textcontrols matching the options.
	opt = options.getFirst();
	while(opt) {
		if(opt->getShowInOptions()) {
			new wxStaticText(this, -1, opt->getName()->c_str(), wxPoint(10,y), wxSize(150,20));

			char buf[256];
			int w = 265;
			
			switch(opt->getType()) {
				case TYPE_FILE:
					sprintf(buf, opt->getFile()->c_str());
					w -= 22;
					break;
				case TYPE_DIR:
					sprintf(buf, opt->getDir()->c_str());
					w -= 22;
					break;
				case TYPE_STR:
					sprintf(buf, opt->getString()->c_str());
					break;
				case TYPE_INT:
					sprintf(buf, "%d", opt->getInt());
					break;
				case TYPE_BOOL:
					sprintf(buf, "%d", opt->getBool());
					break;
				case TYPE_NONE:
				default:
					sprintf(buf, "Unknown type or NONE!");
					break;
			}
			
			ctrl = new wxTextCtrl(this, -1,	buf,
								wxPoint(130,y), wxSize(w,20), 
								wxTAB_TRAVERSAL, wxDefaultValidator,
								opt->getName()->c_str());
			
			switch(opt->getType()) {
				case TYPE_FILE:
					new wxButton(this, BTN_BROWSEFILE, "...", wxPoint((w + 130)+2,y), wxSize(20,20),
							wxTAB_TRAVERSAL, wxDefaultValidator, opt->getName()->c_str());
					break;
				case TYPE_DIR:
					new wxButton(this, BTN_BROWSEDIR, "...", wxPoint((w + 130)+2,y), wxSize(20,20), 
							wxTAB_TRAVERSAL, wxDefaultValidator, opt->getName()->c_str());
					break;
				default:
					break;
			}

			children->Append(ctrl);
			y += 25;
		}
		opt = opt->getNext();
	}

	// Insert the buttons
	btnCancel = new wxButton(this, BTN_CANCEL, "Cancel", wxPoint(225,y), wxSize(80,22), wxTAB_TRAVERSAL);
	btnOk = new wxButton(this, BTN_OK, "Ok", wxPoint(315,y), wxSize(80,22), wxTAB_TRAVERSAL);

	SetDefaultItem(btnOk);
	btnOk->SetFocus();
	
	SetClientSize(400, y + 30);
	Refresh(true);
}

OptionsWindow::~OptionsWindow() 
{AA
}

void OptionsWindow::evtOnBrowseFile(wxCommandEvent& event)
{AA
	wxString name = ((wxButton*)event.GetEventObject())->GetName();
	Option *opt = options.getOption(name.c_str());
	DEBUGPRINTPAR("<-[OptionsWindow::evtOnBrowse]: %s\n", name.c_str())

	wxFileDialog *dlg = new wxFileDialog(this);

	dlg->SetPath(opt->getFile()->c_str());
	
	if(dlg->ShowModal() == wxID_OK) {	
		DEBUGPRINTPAR("<-[OptionsWindow::evtOnBrowse]: Got: %s\n", dlg->GetPath().c_str());
//		opt->setFile((char*)dlg->GetPath().c_str());

		for(unsigned int cnt = 0; cnt < children->GetCount(); cnt++) {
			wxNode *node = children->Item(cnt);
			wxTextCtrl *ctrl = (wxTextCtrl*)node->GetData();
			if(opt->getName()->CompareTo(ctrl->GetName()) == 0) {
				ctrl->SetValue(dlg->GetPath());
			}
		}
	}
}

void OptionsWindow::evtOnBrowseDir(wxCommandEvent& event)
{AA
	wxString name = ((wxButton*)event.GetEventObject())->GetName();
	Option *opt = options.getOption(name.c_str());
	DEBUGPRINTPAR("<-[OptionsWindow::evtOnBrowse]: %s\n", name.c_str())

	wxDirDialog *dlg = new wxDirDialog(this);
	
	dlg->SetPath(opt->getDir()->c_str());
	
	if(dlg->ShowModal() == wxID_OK) {	
		DEBUGPRINTPAR("<-[OptionsWindow::evtOnBrowse]: Got: %s\n", dlg->GetPath().c_str());
//		opt->setDir((char*)dlg->GetPath().c_str());

		for(unsigned int cnt = 0; cnt < children->GetCount(); cnt++) {
			wxNode *node = children->Item(cnt);
			wxTextCtrl *ctrl = (wxTextCtrl*)node->GetData();
			if(opt->getName()->CompareTo(ctrl->GetName()) == 0) {
				ctrl->SetValue(dlg->GetPath());
			}
		}
	}
}

void OptionsWindow::evtOnOk(wxCommandEvent& WXUNUSED(event) )
{AA
	for(unsigned int cnt = 0; cnt < children->GetCount(); cnt++) {
		wxNode *node = children->Item(cnt);
		wxTextCtrl *ctrl = (wxTextCtrl*)node->GetData();
		Option *opt = options.getOption(ctrl->GetName().c_str());
		if(opt->getName()->CompareTo("NULL") != 0) {
			DEBUGPRINTPAR("+ [OptionsWindow::OnOK], Write: %s\n", opt->getName()->c_str())
			wxString strval = ctrl->GetValue();
			long int intval;
			strval.ToLong(&intval);
			
			if(opt->getType() == TYPE_FILE) opt->setFile((char*)strval.c_str());
			if(opt->getType() == TYPE_DIR) opt->setDir((char*)strval.c_str());
			if(opt->getType() == TYPE_STR) opt->setString((char*)strval.c_str());
			if(opt->getType() == TYPE_INT) opt->setInt((int) intval);
			if(opt->getType() == TYPE_BOOL) opt->setBool((bool) intval);
		}
	}
	
	options.save();
//	EndModal(0);
	Show(false);
}

void OptionsWindow::reset()
{AA
	for(unsigned int cnt = 0; cnt < children->GetCount(); cnt++) {
		wxNode *node = children->Item(cnt);
		wxTextCtrl *ctrl = (wxTextCtrl*)node->GetData();
		Option *opt = options.getOption(ctrl->GetName().c_str());

		char buf[256];
		switch(opt->getType()) {
			case TYPE_FILE:
				sprintf(buf, opt->getFile()->c_str());
				break;
			case TYPE_DIR:
				sprintf(buf, opt->getDir()->c_str());
				break;
			case TYPE_STR:
				sprintf(buf, opt->getString()->c_str());
				break;
			case TYPE_INT:
				sprintf(buf, "%d", opt->getInt());
				break;
			case TYPE_BOOL:
				sprintf(buf, "%d", opt->getBool());
				break;
			case TYPE_NONE:
			default:
				sprintf(buf, "Unknown type or NONE!");
				break;
		}
		ctrl->SetValue(buf);
	}
}

void OptionsWindow::evtOnCancel(wxCommandEvent& WXUNUSED(event) )
{AA
//	EndModal(0);
	Show(false);
}
