/*
 * PC-Show - A crossplatform imageviewer, including integrated 
 * functionality with PC-Praxis.
 *
 * Copyright (C) 2004  Bent Bisballe Jensen
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "global.h"

#include "watchfile.h"
#include "options.h"

#include <wx/file.h>

Watchfile::Watchfile()
{AA
	name = NULL;
	path = NULL;
}

Watchfile::~Watchfile()
{AA
	delete name;
	delete path;
}

/** hasFile
 */
bool Watchfile::hasFile()
{
	return path != NULL;
}

/** getName
 * Returns the last read name (must call read before calling this!).
 */
wxString* Watchfile::getName()
{AA
	if(!name) {
		fprintf(stderr, "FATAL: Tried to get name, before reading watchfile!\n");
		exit(1);
	}
	return name;
}

/** getPath
 * Returns the last read path (must call read before calling this!).
 */
wxString* Watchfile::getPath()
{AA
	if(!path) {
		fprintf(stderr, "FATAL: Tried to get path, before reading watchfile!\n");
		exit(1);
	}
	return path;
}

// Returns true if the file has changed since last read.
bool Watchfile::read()
{AA
	wxLogNull noLogging; // Disable noisy errors
	
	Option *optwatch = options.getOption("Paths.watchfile");

	bool changed;
	char *buf;

	wxFile *fp = new wxFile();
	if(!fp->Exists(optwatch->getFile()->c_str())) {
		fprintf(stderr, "Invalid watchfile! [%s]\n", optwatch->getFile()->c_str());
		return false;
	}
	// Read
	fp->Open(optwatch->getFile()->c_str(), wxFile::read);
	int bufsize = fp->SeekEnd() + 1;
	fp->Seek(0);
	buf = (char*)calloc(1, bufsize);
	fp->Read(buf, bufsize);
	fp->Close();

	// Save the old values.
	wxString *oldname = name;
	wxString *oldpath = path;
	
	// Parse
	wxString *filedata = new wxString(buf);
	path = new wxString(filedata->BeforeFirst(' '));
	path->Replace("\\", "/");
	name = new wxString(filedata->AfterFirst(' ').Strip());

	if(oldname && oldpath)
		changed = (*name != *oldname) || (*path != *oldpath);
	else changed = true;
	
	// Clean up
	if(oldname) delete oldname;
	if(oldpath) delete oldpath;
	delete filedata;
	free(buf);

	// Return true if the watchfile has changed.
	return changed;
}
